TASKS = {
    "binary_classification": 1,
    "multi_class_classification": 2,
    "entity_extraction": 4,
    "extractive_question_answering": 5,
    "summarization": 8,
    "single_column_regression": 10,
    "speech_recognition": 11,
}

DATASETS_TASKS = ["text-classification", "question-answering-extractive"]
